	PAGE	,132
	TITLE	RUNPC IBM Interrupt Emulation Program V1.9
;
; Program to emulate some IBM PC interrupt service functions
;
; Copyright 1983 by Lindley Systems
; 21 Hancock St.  Bedford, MA 01730
;
; K. A. Lively	19-May-83 Initial program
; K. D. Dietz	15-Jul-83 adds + fix & debug
; K. A. Lively	11-Aug-83 new scroll routines and added jump table
; K. A. Lively	 4-Sep-83 faster cursor position routine
; R. A. Lindley 24-Sep-83 Added sign on message
; W. J. Lindley 25-Sep-83 Release of v 1.3
; V 1.4 10/1/83 10:15
; V 1.5 10/15/83 Added INT17 service
; V 1.6 12/18/83 Added INT10 function 8 service 19:00
; V 1.8 01/28/84 K. A. Lively  New cursor positioning routine plus
;		 fixes for MTR ROM version 2.5
; V 1.9 11/14/85 Fixed scroll down.
	.XLIST
	.XCREF
	INCLUDE DEFMS.ASM
	INCLUDE DEFCHR.ASM
	.LIST
	.CREF
SAVE_ALL	MACRO
	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX
	PUSH	SI
	PUSH	DI
	ENDM
RESTORE_ALL	MACRO
	POP	DI
	POP	SI
	POP	DX
	POP	CX
	POP	BX
	POP	AX
	ENDM
MTR_VER       EQU  0005H
HORZ_CHAR     EQU  0291H
;
RUNPC  SEGMENT
	ASSUME	CS:RUNPC, SS:RUNPC, DS:RUNPC, ES:NOTHING
	ORG 	100H
;
FLABEL: JMP	INIT
;
; INTERRUPT 10, ENTRY POINT.
INT10:	PUSH	DS
	PUSH	CS
	POP	DS
	PUSH	BX
	STI
	PUSH	AX
	MOV	AL,008H 	; VRAM PARALLELED
	OUT	0D8H,AL
	POP	AX
	MOV	BL,AH		;get fct code to bx
	XOR	BH,BH
	CMP	BX,15		;check for in range
	JG	OOPS		;no service if not
	SHL	BX,1		;make into word offset
	JMP	WORD PTR JMP_TBL[BX]	;case to function
JMP_TBL:
	DW	OFFSET	SET_MODE	;ah=0  set video mode
	DW	OFFSET	SET_CUR_TYPE	;ah=1  set cursor type
	DW	OFFSET	SET_CUR_POS	;ah=2  set cursor position
	DW	OFFSET	GET_CUR 	;ah=3  read cursor position
	DW	OFFSET	OOPS		;ah=4  read light pen position
	DW	OFFSET	OOPS		;ah=5  select active display page
	DW	OFFSET	SCROLL_UP	;ah=6  scroll active page up
	DW	OFFSET	SCROLL_DWN	;ah=7  scroll active page down
	DW	OFFSET	READ_CHR	;ah=8  read char and attrib. at cursor
	DW	OFFSET	WCHAR_ATTR	;ah=9  write char and attrib. at cursor
	DW	OFFSET	WCHAR		;ah=10 write char only at cursor
	DW	OFFSET	OOPS		;ah=11 set color palette
	DW	OFFSET	OOPS		;ah=12 write dot
	DW	OFFSET	OOPS		;ah=13 read dot
	DW	OFFSET	WCHTTY		;ah=14 write text in teletype mode
	DW	OFFSET	GET_MODE	;ah=15
; SORRY, NO SERVICE FOR THIS FUNCTION
OOPS:
	SAVE_ALL
	MOV	BX,OFFSET OOPS_MSG
	CALL	CON_OUT
	MOV	DH,AH
	CALL	ONEHEX
	CALL	ONEHEX
	MOV	BX,OFFSET CR_LF
	CALL	CON_OUT
	RESTORE_ALL
EXIT:
	POP	BX
	POP	DS
	IRET
;
; INTERRUPT 10, FUNCTION 0, SET VIDEO MODE
SET_MODE:	; HEATH/ZENITH ONLY HAS ONE MODE
	JMP	EXIT

; INTERRUPT 10, FUNCTION 1, SET CURSOR LINES
; For the set cursor type routine, no actual setting of the cursor is
; performed due to differing numbers of scan lines, but the cursor mode
; is remembered so that it can be supplied in the get cursor position
; routine.
SET_CUR_TYPE:
	MOV	WORD PTR CURSOR_MODE,CX
	JMP	EXIT
;
; INTERRUPT 10, FUNCTION 2, SET CURSOR POSITION
SET_CUR_POS:
	PUSH	ES		;save registers
	PUSH	AX
	PUSH	DX
	PUSH	CX
	MOV	AX,WORD PTR MTR ;segment address of monitor data segment
	MOV	ES,AX
	CMP	DL,79		;check for max column
	JLE	SCP_A		;was ok, continue
	MOV	DL,79		;too big, set to col 79
SCP_A:	MOV	BX,HORZ_CHAR	;point to cursor column
	MOV	ES:[BX],DL	;store new column
	INC	BX		;point to cursor row
	CMP	DH,24		;check for max row
	JLE	SCP_B		;ok, continue
	MOV	DH,24		;too big, set to row 24
SCP_B:	MOV	ES:[BX],DH	;store new row
	MOV	AL,DH		;calculate new cursor start address
	MOV	CL,5
	MUL	CL
	MOV	CL,4
	SHL	AX,CL
	MOV	BX,CRTC_DISPLAY
	ADD	AX,ES:[BX]
	MOV	CL,DL
	XOR	CH,CH
	ADD	AX,CX
	MOV	BX,CRTC_CURSOR
	MOV	ES:[BX],AX		;store new cursor start address
	MOV	BX,CRSR_UPDATE
	MOV	BYTE PTR ES:[BX],0FFH	;force update on vert retrace
	POP	CX			;recover registers
	POP	DX
	POP	AX
	POP	ES
	JMP	EXIT
;
; INTERRUPT 10, FUNCTION 3, READ CURSOR POSITION
GET_CUR:			;same comment as above regarding version
	PUSH	ES		;  dependence
	PUSH	AX
	MOV	AX,WORD PTR MTR ;point to monitor data segment
	MOV	ES,AX
	MOV	BX,HORZ_CHAR	;point to cursor column
	MOV	DL,ES:[BX]	;put in dl
	INC	BX		;point to cursor row
	MOV	DH,ES:[BX]	;put in dh
	MOV	CX,WORD PTR CURSOR_MODE ;put cursor mode in cx
	POP	AX
	POP	ES
	JMP	EXIT
;
; INTERRUPT 10, FUNCTION 6, SCROLL ACTIVE PAGE UP
SCROLL_UP:
	SAVE_ALL
	CALL	SCR_PARAM
	MOV	BH,TOP_LINE
	MOV	BL,LFT_CHAR
	CALL	CCA		; TOP LINE OF WINDOW
	MOV	DI,BX
	MOV	BH,TOP_LINE
	ADD	BH,SCR_LINE
	CALL	CCA		; TOP LINE TO SCROLL UP
	MOV	SI,BX
	MOV	AH,HGT_WNDO
	SUB	AH,SCR_LINE
	MOV	DL,QTY_CHAR
	XOR	DH,DH
	MOV	AL,SCR_LINE
	PUSH	DS
	PUSH	ES
	MOV	BX,0E000H	; GREEN RAM SEGMENT
	MOV	DS,BX
	MOV	ES,BX
	PUSH	AX
UP_LOOP:
	OR	AH,AH
	JZ	UP_BLANK
	CALL	MDL
	ADD	SI,2048
	ADD	DI,2048
	DEC	AH
	JNZ	UP_LOOP
UP_BLANK:
	POP	AX
	MOV	AH,AL
UP_B1:	OR	AH,AH
	JZ	UP_B2
	CALL	BDL	; BLANK ONE LINE
	ADD	DI,2048
	DEC	AH
	JNZ	UP_B1
UP_B2:	POP	ES
	POP	DS
	RESTORE_ALL
	JMP	EXIT
;
; INTERRUPT 10, FUNCTION 7, SCROLL ACTIVE PAGE DOWN
SCROLL_DWN:
	SAVE_ALL
	CALL	SCR_PARAM
	MOV	BH,BTM_LINE
	MOV	BL,LFT_CHAR
	CALL	CCA		; DESTINATION LINE OFFSET
	MOV	DI,BX
	MOV	BH,BTM_LINE
	SUB	BH,SCR_LINE
	CALL	CCA		; SOURCE LINE OFFSET
	MOV	SI,BX
	MOV	AH,HGT_WNDO
	SUB	AH,SCR_LINE
	MOV	DL,QTY_CHAR
	XOR	DH,DH
	MOV	AL,SCR_LINE
	PUSH	DS
	PUSH	ES
	MOV	BX,0E000h	; GREEN RAM SEGMENT
	MOV	DS,BX
	MOV	ES,BX
	PUSH	AX
DN_LOOP:
	OR	AH,AH
	JZ	DN_BLANK
	CALL	MDL			;move the display line
	SUB	DI,2048 		;else decrement the line pointers
	SUB	SI,2048
	DEC	AH
	JNZ	DN_LOOP 	       ;and do it again
DN_BLANK:
	POP	AX
	MOV	AH,AL
DN_B1:	OR	AH,AH
	JZ	DN_B2
	CALL	BDL	; BLANK ONE LINE
	SUB	DI,2048
	DEC	AH
	JNZ	DN_B1
DN_B2:	POP	ES
	POP	DS
	RESTORE_ALL
	JMP	EXIT
; PROC TO EXTRACT AND VERIFY SCROLL PARAMETERS
; INPUTS:
;	DH	BOTTOM LINE
;	DL	RIGHT COLUMN
;	CH	TOP LINE
;	CL	LEFT COLUMN
;	BH	ATTRIBUTE (NOT USED)
;	AL	NUMBER OF LINES TO SCROLL
SCR_PARAM PROC NEAR
	CMP	DH,24	; TEST BOTTOM LINE
	JBE	SCR1
	MOV	DH,24
SCR1:	MOV	BTM_LINE,DH
	CMP	CH,DH	; TOP LINE MUST BE <= BOTTOM LINE
	JBE	SCR2
	MOV	CH,DH
SCR2:	MOV	TOP_LINE,CH
	CMP	DL,79	; RIGHT CHAR
	JBE	SCR3
	MOV	DL,79
SCR3:	CMP	CL,DL	; LEFT CHAR MUST BE <= RIGHT CHAR
	JBE	SCR4
	MOV	CL,DL
SCR4:	MOV	LFT_CHAR,CL
	SUB	DL,CL
	INC	DL
	MOV	QTY_CHAR,DL
	SUB	DH,CH	; LINES IN WINDOW
	INC	DH
	MOV	HGT_WNDO,DH
	OR	AL,AL	; ZERO BLANKS ALL LINES
	JNZ	SCR5
	MOV	AL,DH
SCR5:	CMP	AL,DH
	JBE	SCR6
	MOV	AL,DH
SCR6:	MOV	SCR_LINE,AL
	RET
SCR_PARAM ENDP
;
; CALCULATE CHARACTER ADDRESS OFFSET IN VIDEO RAM
CCA	PROC	NEAR
	SHL	BH,1	;enter with bh,bl = row,col of char
	SHL	BH,1	;exit with bx = offset
	SHL	BH,1	;uses bh
	RET
CCA	ENDP
; MOVE ONE ROW OF VIDEO, CHARACTERS, AND ATTRIBUTES
MDL	PROC	NEAR
	PUSH	AX
	PUSH	DI
	PUSH	SI
	MOV	CX,11
	CLD		;enter with di -> destination line
MDL1:	PUSH	CX	;    si -> source line
	PUSH	DI	;    cx = 11 (scan lines per char)
	PUSH	SI	;    ds = es = E000H
	MOV	CX,DX	;    DX = no. of chars per line
	XOR	CH,CH
	SHR	CX,1
	JNC	MDL2
	MOVSB
MDL2:	REP	MOVSW
	POP	SI
	POP	DI
	POP	CX
	ADD	DI,128
	ADD	SI,128
	LOOP	MDL1
	POP	SI
	POP	DI
	POP	AX
	RET
MDL	ENDP
;
; BLANK A DISPLAY LINE
BDL	PROC	NEAR
	PUSH	DI
	CLD		;enter with same register set-up as
	MOV	CX,11
	XOR	AL,AL
BDL1:	PUSH	CX	;  above except that si is not used
	PUSH	DI
	MOV	CX,DX	;  DX = no. of chars per line
	XOR	CH,CH
	REP	STOSB
	POP	DI
	POP	CX
	ADD	DI,128
	LOOP	BDL1
	POP	DI
	RET
BDL	ENDP
; INTERRUPT 10, FUNCTION  8, READ CHARACTER & ATTRIBUTES AT CURSOR
;
READ_CHR:
	PUSH	AX
	PUSH	ES
	MOV	AX,WORD PTR MTR ;point to monitor data segment
	MOV	ES,AX
	MOV	BX,HORZ_CHAR	;point to cursor column & ROW
	MOV	BX,ES:[BX]
	MOV	AX,0E000H	; GREEN RAM ADDRESS
	MOV	ES,AX
	CALL	CCA
	MOV	AL,ES:BYTE PTR [BX+9*128]	; FONT INDEX
	MOV	AH,ES:BYTE PTR [BX+10*128]	; ATTRIBUTES
	AND	AH,080H ; REVERSE VIDEO IF SET
	MOV	AH,007H ; NORMAL VIDEO
	JZ	R_C_N
	MOV	AH,070H ; REVERSE VIDEO
R_C_N:	POP	ES
	POP	AX
	JMP	EXIT
; INTERRUPT 10, FUNCTION 9, WRITE CHARACTER WITH ATTRIBUTES AT CURSOR
WCHAR_ATTR:
	POP	BX		; RESTORE ATTRIBUTES
	PUSH	BX
	PUSH	ES
	MOV	BH,BL
	AND	BH,01110110B	; LOOK FOR NON DISPLAY
	JNZ	NOT_BLANKED
	MOV	AL,32		; BLANK SPACE
	JMP	NORM_CHAR
NOT_BLANKED:
	CMP	BL,00000111B	; NORMAL
	JZ	NORM_CHAR
	MOV	BH,BL
	AND	BH,00001000B
	JNZ	NORM_CHAR
SET_REVERSE:
	MOV	BYTE PTR REV_FLAG,0FFH	; SET LOCAL FLAG
	PUSH	AX
	MOV	AX,WORD PTR MTR 	; AND BIOS MODE FLAG
	MOV	ES,AX
	MOV	BX,H19_REV
	MOV	WORD PTR ES:[BX],0FFFFH
	POP	AX
NORM_CHAR:		; CHARACTER IN (AL) TO SCREEN
	PUSH	CX
WCHAR_ATTR2:
	CALL	FAR PTR BIOS_CONOUT	;have the bios put out the char
	LOOP	WCHAR_ATTR2		;  and do cx copies
	MOV	AL,8
	CALL	FAR PTR BIOS_CONOUT
	POP	CX
	CMP	BYTE PTR REV_FLAG,0FFH	;did we set the local flag?
	JNZ	WCHAR_ATTR3		;no, skip
	MOV	BYTE PTR REV_FLAG,0	;else restore flags
	MOV	BX,H19_REV
	MOV	WORD PTR ES:[BX],0
WCHAR_ATTR3:
	POP	ES
	JMP	EXIT
; INTERRUPT 10, FUNCTION 14, WRITE TO SCREEN & ADVANCE CURSOR
WCHTTY: MOV	CX,1
WCHAR:	PUSH	CX
WCHAR1: CALL	FAR PTR BIOS_CONOUT  ;write cx copies of char in al
	LOOP	WCHAR1		     ;	to console
	POP	CX
	JMP	EXIT
; INTERRUPT 10, FUNCTION 15, READ VIDEO MODE
GET_MODE:
	MOV	AX,5002H	; 80 X 25 B/W, ALPHA DISPLAY
	POP	BX		;get back original bx to show page 0
	MOV	BH,00
	PUSH	BX		;  and then put it back on the stack
	JMP	EXIT

; INTERRUPT 11, EQUIPMENT CHECK
; BITS 15,14  : 0 : ATTACHED PRINTERS
;     13     : 0 : NOT USED
;     12     : 0 : GAME IO ATTACHED
;     11,10,9: 0 : NUMBER OF 232C CARDS ATTACHED
;      8     : 0 : NOT USED
;      7,6   :01 : 2 DISK DRIVES
;      5,4   :11 : B/W MONITOR
;      3,2   :11 : 64K ON PLANAR
;      1     : 0 : NOT USED
;      0     : 1 : IPL FROM DISKETTE
;     15-0   : 007DH
INT11:	MOV	AX, 007DH
	IRET
; INTERRUPT 12, MEMORY SIZE, ALWAYS 64K
INT12:	MOV	AX, 64
	IRET
; INTERRUPT 16, KEYBOARD OPERATIONS
;   THIS IS A PARTIAL IMPLEMENTATION
INT16	PROC	FAR
	STI
	OR	AH,AH
	JNZ	INT16A
; AH=0	SCAN CODE TO AH & CHARACTER TO AL
	MOV	AH,7
	INT	21H
	XOR	AH,AH	; SCAN CODE NOT SUPPORTED
	IRET
INT16A: DEC	AH
	JNZ	INT16B
; AH=1	RETURN KEYBOARD STATUS IN ZF
;	CHARACTER IN AX & REMAINS IN BUFFER
	CALL	FAR PTR BIOS_STATUS
	RET	2
; AH=2	KEYBOARD STATUS ONLY
INT16B: XOR	AX,AX
	IRET
INT16	ENDP
; INTERRUPT 17, PRINTER SERVICE
INT17:	STI
	PUSH	AX
	PUSH	DX
	OR	AH,AH	; 0 => PRINT (AL)
	MOV	DL,AL
	JNZ	INT17A
	PUSH	AX
	MOV	AH,5
	INT	21H
	POP	AX
; PRINTER STATUS EXPECTED IN AH
; BIT 7 1 = READY
;     6 1 = ACKNOWLEDGE
;     5 1 = OUT OF PAPER
;     4 1 = ON LINE
;     3 1 = PRINT ERROR
;     2     NOT USED
;     1     NOT USED
;     0 1 = TIMED OUT
INT17A: POP	DX
	POP	AX
	MOV	AH,080H ; ALWAYS A-OK
	IRET
;
; INTERRUPT 21, PROGRAM TERMINATION
INT21:
	OR	AH,AH
	JNZ	SHORT XIT_21
	PUSH	DS
	PUSH	CS
	POP	DS
	PUSH	BX
	MOV	BX,OFFSET XIT_25TH
	CALL	CON_OUT
	POP	BX
	POP	DS
XIT_21: DB	0EAH	; JMP INTER-SEGMENT
DOS_OFF DW     0
DOS_SEG DW     0
;
; CHARACTERS TO SCREEN
; [BX] -> CHAR STRING WITH NULL END SIGNAL
CON_OUT PROC	NEAR
	PUSH	AX
LOOP:	MOV	AL,[BX]
	OR	AL,AL
	JZ	DONE
	CALL	FAR PTR BIOS_CONOUT
	INC	BX
	JMP	LOOP
DONE:	POP	AX
	RET
CON_OUT ENDP
ONEHEX PROC NEAR
	PUSH	AX
	PUSH	CX
	MOV	CL,4	; TOP 4 BITS TO LOW END
	ROL	DX,CL
	MOV	AL,DL
	AND	AL,0FH	; ONLY LOW BITS WANTED
	DAA		; IF A TO F, ADD 6
	ADD	AL,0F0H ; TAKE CARRY BITS OUT
	ADC	AL,040H ; MAKE INTO ASCII CHAR
	CALL	FAR PTR BIOS_CONOUT
	POP	CX
	POP	AX
	RET
ONEHEX	ENDP
; Data Space
TOP_LINE DB	0
BTM_LINE DB	0
LFT_CHAR DB	0
QTY_CHAR DB	0
SCR_LINE DB	0
HGT_WNDO DB	0
CURSOR_MODE	DW	0
MTR		DW	0	;MTR-100 DATA SPACE POINTER
H19_REV 	DW	02D8H	;the following are for MTR version 2.5
CRTC_CURSOR	DW	02BAH	;  will be modified at start-up if MTR
CRSR_UPDATE	DW	02BCH	;  is a lower version
CRTC_DISPLAY	DW	02BDH
REV_FLAG	DB	0
CR_LF	DB	13,10,0
CLS	DB	27,'E',0
ON_25TH DB	27,'x1',0
CLR_25TH DB	27,'y1',27,'x1',0
XIT_25TH DB	27,'y1',27,'x1',27,'Y7 ',27,'K',27,'Y6 ',0
GET_CUR_CHAR DB 27,'_',0
OOPS_MSG LABEL NEAR
	DB	13,10,' *** NO SERVICE FOR FUNCTION',32,0

END_PROGRAM LABEL NEAR

INIT:	XOR	AX,AX
        MOV	ES,AX 			;ACCESS INTERRUPT PAGE
	MOV	AX,ES:(021H*4)		; GET OFFSET ADDRESS
	MOV	DOS_OFF,AX
	MOV	AX,ES:(021H*4+2)	; GET SEGMENT ADDRESS
	MOV	DOS_SEG,AX
	MOV	ES:(010H*4+2), CS	;SET SEGMENT PART OF ADDRESS
	MOV	ES:(011H*4+2), CS
	MOV	ES:(012H*4+2), CS
	MOV	ES:(016H*4+2), CS
	MOV	ES:(017H*4+2), CS
	MOV	ES:(021H*4+2), CS
	MOV	AX, OFFSET INT10	; VIDEO CONTROL
	MOV	ES:(010H*4), AX
	MOV	AX, OFFSET INT11	;EQUIPMENT CHECK
	MOV	ES:(011H*4), AX
	MOV	AX, OFFSET INT12	;MEMORY SIZE, 192K
	MOV	ES:(012H*4), AX
	MOV	AX, OFFSET INT16	;KEYBOARD I/O
	MOV	ES:(016H*4), AX
	MOV	AX, OFFSET INT17	;PRINTER OUT
	MOV	ES:(017H*4), AX
	MOV	AX, OFFSET INT21	; PROGRAM TERMINATION
	MOV	ES:(021H*4), AX
	MOV	AX,ES:03FEH
	MOV	WORD PTR MTR,AX
	MOV	ES,AX
	MOV	AL,BYTE PTR ES:MTR_VER
	CMP	AL,25H			;do we have version 2.5?
	JZ	INIT_1			;yes, skip
	MOV	WORD PTR H19_REV,02BEH		;else change offsets for ver 1.x
	MOV	WORD PTR CRTC_CURSOR,02A2H
	MOV	WORD PTR CRSR_UPDATE,02A4H
	MOV	WORD PTR CRTC_DISPLAY,02A5H
INIT_1: IN	AL,0D8H 		;ENSURE THAT THE CPU HAS
	AND	AL,07FH 		;  ACCESS TO VIDEO RAM BY
	OUT	0D8H,AL 		;  CLEARING BIT 7 AT PORT D8
	MOV	BX,OFFSET ON_25TH
	CALL	CON_OUT
	MOV	BX,OFFSET SIGN_ON
	CALL	CON_OUT
	MOV	DX, OFFSET END_PROGRAM	;SET UP FOR INT 27H
	INT	27H			;TERMINATE BUT STAY RESIDENT
;
SIGN_ON DB	27,69,'RUNPC V1.9 Emulates some IBM PC interrupts.',13,10
	DB	'(C)1983 by Lindley Systems 11/14/85',13,10,0
RUNPC	ENDS
	END	FLABEL
