	PAGE	,132
	TITLE	SCREEN - Screen Dump Utility  Version 1.8
;
;	Kenneth A. Lively   5 November 1984
;
;	COPYRIGHT (C) 1984 LINDLEY SYSTEMS
;	Clear 25th line if on. RAL 2/2/85
;	Added IBM type printer option. RAL 12/2/85 V1.8
;
;	SCREEN is a screen dump utility. It intercepts INT 5 from
;	the BIOS (generated when <shift f12> is struck), and dumps
;	the contents of the screen onto the PRN device.
;	Although any color combination can be dumped, depending on the
;	value of the variable COLOR, the program setscrn.com is respon-
;	sible for making sure that video ram is fully populated before
;	allowing a color other than green to be chosen.
;
FALSE	EQU	0
TRUE	EQU	NOT FALSE
;
; Set ONLY ONE of the following to TRUE and all others to FALSE
FX80	EQU	FALSE	; the Epson FX-80 or RX-80
MX80	EQU	FALSE	; the MX-80 with GRAFTRAX
IBM	EQU	TRUE	; IBM version of MX-80 with GRAFTRAX
NEC	EQU	FALSE	; NEC PC 8023A-C and CITOH 8510
MPI	EQU	FALSE	; MPI 88 or 99 printers
TI855	EQU	FALSE	; TI-855 Printer in data mode
TI855WP EQU	FALSE	; TI-855 Printer in word processing mode
;
;  Microline version is in a seperate file, SCRNMLXX.ASM
;
BIOS_SEG SEGMENT AT 40H
	ORG 4*3
	BIOS_PRINT	LABEL FAR
BIOS_SEG  ENDS
;
CODE	SEGMENT	BYTE PUBLIC 'CODE'
	ASSUME	CS:CODE,DS:CODE,ES:CODE,SS:CODE
;
	ORG	100H
BEGIN:	JMP	START
	DW	VERSION
;
;	INT 5 handler
INT_5:	PUSH	DS			; Save his data segment
	PUSH	AX
	MOV	CS:INT_SP,SP
	MOV	CS:INT_SS,SS		; Save his stack
	MOV	AX,CS
	MOV	SS,AX			; Set local stack
	MOV	SP,OFFSET INT_STACK
	STI				; Now allow interrupts to come in
	PUSH	ES
	PUSH	BX
	PUSH	CX
	PUSH	DX
	PUSH	SI
	PUSH	DI
	MOV	DS,AX			; Set up my segments
	MOV	AX,0E000H
	MOV	ES,AX			; Note that ES points to green ram
	CALL	INT_INIT		; Initialize variables and printer
	CALL	INT_SCRN		; Process the screen
	POP	DI
	POP	SI
	POP	DX
	POP	CX
	POP	BX
	POP	ES
	CLI				; INTs off while messing around
	MOV	AX,INT_SP
	MOV	SP,AX
	MOV	AX,INT_SS		; Restore his stack
	MOV	SS,AX
	POP	AX
	POP	DS
	IRET				;back to BIOS
; data space
INT_SS	DW	0			;saved stack values
INT_SP	DW	0
SCRN	DB	320 DUP(?)		;4 scan lines worth of screen bytes
ADDR	DW	4 DUP(?)		;addresses of the 4 scan lines
S_LINE	DB	8			;scan line counter
PRNTR	DB	8 DUP(?)		;bytes to be sent to the printer
ROW	DB	?			;row counter (9 scan lines = one row)
VERSION DB	17			;current version
; The next 4 variables control the portion of the screen to be dumped and
; are not changed by the program.  This allows for a default screen dump
; of the entire screen.  Use SETSCRN to modify the memory resident code
; or change them under program control using the pointer at the INT 5 locn
; in absolute segment 0 to find their location.
FIRST_ROW  DB	0			;first row in the dump (starts at 0)
LAST_ROW   DB	24			;last row PLUS one
FIRST_COL  DB	0			;first column in the dump (starts at 0)
LAST_COL   DB	79			;last column PLUS one
COLOR	   DB	4			;default color is green bank
FIRST_BYTE DB	0			;first byte for the current line
LAST_BYTE  DB	0			;last byte +1 of the current line
FLAG	   DB	?			;end of screen flag
ZEROES	   DB	?			;keeps track of the no. of zero scan
					;  lines needed for doing the last
					;  row of the screen dump
MODE	   DB	?			;the CRT-C control byte storage
ERR_FLAG   DB	0			;error flag for (fatal) internal errors
CR_LF	   DB	0DH,0AH
ERR_MSG    DB	0DH,0AH,'Internal error in screen dump logic!!',0DH
	   DB	0AH,0AH,'$'
; Clears line 25, if it was off
SET_25	   DB	1BH,'x1',1BH,'j',1BH,'Y8 ',1BH,'k$'
;
; The following are the printer dependent graphics mode and line spacing codes
 IF TI855WP
LD_STRG    DB	1BH,1CH,11H,		;16/144 Line spacing
GR_STRG    DB	1BH,'j',?,?		;960 dot graphics
END_STRG   DB	1BH,1EH,9		;back to std spacing (6 LPI)
GR_OFFSET  EQU	2
 ENDIF
 IF FX80
LD_STRG    DB	1BH,'A',8		;line spacing to 8/72's
GR_STRG    DB	1BH,'*',4,?,?		;CRT graphics mode
END_STRG   DB	1BH,'2' 		;back to std spacing
GR_OFFSET  EQU	3
 ENDIF
 IF MX80 OR TI855
LD_STRG    DB	1BH,'A',8		;line spacing to 8/72's
GR_STRG    DB	1BH,'L',?,?		;960 dot graphics mode
END_STRG   DB	1BH,'2' 		;back to std spacing
GR_OFFSET  EQU	2
 ENDIF
 IF IBM 				; Added in V1.8
LD_STRG    DB	1BH,'A',8,1BH,'2'	;line spacing to 8/72's
GR_STRG    DB	1BH,'L',?,?		;960 dot graphics mode
END_STRG   DB	1BH,'A',12,1BH,'2'	;back to std spacing
GR_OFFSET  EQU	2
 ENDIF
 IF NEC
LD_STRG    DB	1BH,'T16',1BH,'N'	;16/144 line spacing, pica mode
	   DB	1BH,'[' 		;incremental mode for alignment
GR_STRG    DB	1BH,'S0',?,?,?
END_STRG   DB	1BH,'A',1BH,']' 	;back to std spacing and logic
					;  seeking mode
DIVISORS   DB	100,10,1		;used for decimal divison
 ENDIF
 IF MPI
LD_STRG    DB	1BH,1FH 		;put in 132 col mode for 82dpi
GR_STRG    DB	1BH,17H 		;enter graphics mode
END_STRG   DB	1BH,1CH 		;back to 80 column mode
 ENDIF
;
	DW	128 DUP (?)
INT_STACK LABEL NEAR		; Internal stack
INT_INIT  PROC	NEAR
;	MOV	AH,9		; Send to screen (V 1.7)
;	MOV	DX,OFFSET SET_25 ; Clears 25th line if off, else does nothing
;	INT	21H
; Each call to get_bytes expects addr[6] to point to the previous scan line,
; so to prepare for the FIRST call, need to set addr[6] to n*2048-1024 where
; n = the first row of the dump.  Also need to set row equal to one less than
; the actual first row and scan_line = 8
	MOV	AX,-1024		;init last scan line offset to -1024
	MOV	CL,FIRST_ROW		;get the first row of the dump to CL
	MOV	ROW,CL			;  and store this value -1
	DEC	ROW
	XOR	CH,CH
INIT_1: JCXZ	INIT_2			;if cl=0, then are all set, else
	ADD	AX,2048 		;add 2048 for each row past first
	DEC	CL
	JMP	SHORT INIT_1
INIT_2: 
IF MPI
	MOV	ADDR[4],AX
ELSE
	MOV	ADDR[6], AX
ENDIF
	MOV	FLAG,0			;flag to zero for start
	MOV	ZEROES,0		;use all screen bytes initially
	MOV	S_LINE,8
	MOV	BX,OFFSET LD_STRG	;set up line spacing and graphics mode
IF FX80 OR MX80 OR TI855 OR TI855WP
	MOV	CX,3
ENDIF
IF IBM
	MOV	CX,5
ENDIF
IF NEC
	MOV	CX,8
ENDIF
IF MPI
	MOV	CX,2
ENDIF
	CALL	PRN_OUT 		;send the string to the printer
	IN	AL,0D8H 		;ensure that the cpu can access
	MOV	MODE,AL 		;  the video ram, but....
	AND	AL,7FH			;  keep a copy of the original
	OUT	0D8H,AL 		;  so we can put it back
	RET
INT_INIT  ENDP
;
INT_SCRN  PROC	NEAR			;do the screen dump
SCRN_0: MOV	AL,FIRST_COL		;reset first_byte to the first col
	MOV	FIRST_BYTE,AL
	MOV	LAST_BYTE,0		;reset last_byte to zero
	CALL	GET_BYTES		;get 4 scan lines worth of bytes
	CMP	FLAG,1			;are we done?
	JNZ	SCRN_1			;no, continue
	JMP	SCRN_DONE		;yes, finish up and exit
SCRN_1: CMP	LAST_BYTE,0		;do we have a blank line?
	JNZ	SCRN_2			;no, continue
IF MPI
	MOV	BX,OFFSET GR_STRG
	MOV	CX,2
	CALL	PRN_OUT
ENDIF
	JMP	SCRN_7			;yes, send out a cr_lf
SCRN_2: CALL	FIND_FIRST		;not blank - find first non-zero data
	CMP	ERR_FLAG,1		;err_flag set if find_first has a bug
	JNZ	SCRN_2A
	JMP	SCRN_DONE		;exit when err_flag is set
SCRN_2A:
IF MPI
	MOV	BX,OFFSET GR_STRG
	MOV	CX,2
	CALL	PRN_OUT
ENDIF
	CMP	FIRST_BYTE,0		;do we send any spaces?
	JZ	SCRN_2B 		;no, process the line
	MOV	CL,FIRST_BYTE		;yes, send them out
IF MPI
	MOV	AX,8
	IMUL	CL
	MOV	CX,AX
	MOV	AL,40H
ELSE
	XOR	CH,CH
	MOV	AL,' '
ENDIF
SPACES: CALL	FAR PTR BIOS_PRINT
	LOOP	SPACES
SCRN_2B:
IF NOT MPI
	CALL	GR_CALC 		;calc the no. of bytes in prntr format
IF FX80
	MOV	CX,5
ENDIF
IF MX80 OR TI855 OR TI855WP OR IBM
	MOV	CX,4
ENDIF
IF NEC
	MOV	CX,6
ENDIF
	MOV	BX,OFFSET GR_STRG	;send the graphics mode string
	CALL	PRN_OUT
ENDIF
	MOV	BL,FIRST_BYTE		;start the conversion process that
					;converts the horizontally oriented
					;screen bytes to vertically oriented
					;bytes for the printer
	XOR	BH,BH			;BX is the column index
SCRN_3: MOV	SI,OFFSET SCRN		;SI is offset to the screen bytes
	MOV	DI,OFFSET PRNTR 	;first, zero the destination bytes
	MOV	CX,8
SCRN_4: MOV	BYTE PTR[DI],0
	INC	DI
	LOOP	SCRN_4
SCRN_5: MOV	DI,OFFSET PRNTR 	;point to bytes to be formed
	MOV	CH,8			;  and set counter
SCRN_6: MOV	AL,[SI][BX]		;get a scrn byte
	AND	AL,128			;only use the top bit
IF NEC
	ROL	AL,1			;for the CITOH, LSB is at the top
ENDIF
	PUSH	CX			;save the counters
	SAL	CL,1			;shift the scrn byte left by two
					;  times the scan line index
IF NEC
	SHL	AL,CL			;shift left for the CITOH
	MOV	AH,AL
	SHL	AH,1
ELSE
	SHR	AL,CL			;and right for the Epson printers
	MOV	AH,AL
	SHR	AH,1
ENDIF
	OR	AL,AH			;now OR everything together
	OR	AL,[DI] 		; and store the result
	MOV	[DI],AL
	SAL	BYTE PTR[SI][BX],1	;shift the scrn byte for next time
	INC	DI			;set up to work on the next prntr byte
	POP	CX			;get back counters
	DEC	CH			;  and decrement inner loop counter
	JNZ	SCRN_6			;repeat for all eight prntr bytes
	ADD	SI,80			;now set up to do next scan line
	INC	CL			;increment scan line ctr
IF MPI
	CMP	CL,3
ELSE
	CMP	CL,4			;are we done with this set of 4
ENDIF
	JNZ	SCRN_5			;  screen bytes?
	PUSH	BX
	CALL	OUT_BYTES		;yes, send out the prntr bytes
	POP	BX
	INC	BX			;increment the column counter
	CMP	BL,LAST_BYTE		;done with this line?
	JZ	SCRN_7			;yes, send cr_lf, and start the next
	JMP	SCRN_3			;no, continue with this line
SCRN_7: 
IF MPI
	MOV	AL,36H
	CALL	FAR PTR BIOS_PRINT
ELSE
	MOV	CX,2			;send the cr_lf
	MOV	BX,OFFSET CR_LF
	CALL	PRN_OUT
ENDIF
	JMP	SCRN_0			;start the next line
SCRN_DONE:				;arrive here when done with the screen
	MOV	BX,OFFSET END_STRG	;point to string to reset printer
IF FX80 OR MX80 OR MPI OR TI855
	MOV	CX,2
ENDIF
IF IBM
	MOV	CX,5
ENDIF
IF TI855WP
	MOV	CX,3
ENDIF
IF NEC
	MOV	CX,4
ENDIF
	CALL	PRN_OUT 		;  and send it out
	MOV	AL,MODE 		;all done, so put back the control
	OUT	0D8H,AL 		;  word at port D8
	RET				;  and return
INT_SCRN  ENDP
;
FIND_FIRST PROC NEAR			;find the first non-zero data in the
	MOV	BL,FIRST_BYTE		;  current line - BX is column counter
	XOR	BH,BH
FIND_0: MOV	SI,OFFSET SCRN		;point to the screen bytes
IF MPI
	MOV	CX,3
ELSE
	MOV	CX,4			;check all scan lines, column by column
ENDIF
FIND_1: CMP	BYTE PTR [SI][BX],0	;still zero data?
	JZ	FIND_2			;yes, check next line of column
	MOV	FIRST_BYTE,BL		;no, set first_byte to this col and ret
	RET
FIND_2: ADD	SI,80			;look at next line
	LOOP	FIND_1			;  and do it again
	INC	BX			;increment column count
	CMP	BL,LAST_BYTE		;is the whole line blank???
	JNZ	FIND_0			;no, continue
	MOV	DX,OFFSET ERR_MSG	;yes, a fatal error since get_bytes
	MOV	AH,9			;  should have found this out already
	INT	21H
	MOV	ERR_FLAG,1		;show fatal error
	RET
FIND_FIRST ENDP
;
PRN_OUT  PROC	NEAR			;output the number of bytes in cx
P_LOOP: MOV	AL,[BX]
	CALL	FAR PTR BIOS_PRINT
	INC	BX
	LOOP	P_LOOP
	RET
PRN_OUT  ENDP
;
GET_BYTES  PROC  NEAR			;get 4 scan lines worth of screen
	CMP	ZEROES,0		;see if we are doing odd scan lines
	JZ	GET_0			;  at the end of the screen
	MOV	FLAG,1			;yes, then return
	RET
GET_0:	
IF MPI
	MOV	SI,OFFSET ADDR[4]
ELSE
	MOV	SI,OFFSET ADDR[6]	;set up to increment scan line ptrs
ENDIF
	MOV	DI,OFFSET ADDR
	CALL	CHK_LINE		;go do the hard stuff
	CMP	FLAG,0			;have we finished the whole screen?
	JZ	GET_1			;no, continue
	RET
GET_1:	
IF MPI
	MOV	CX,2
ELSE
	MOV	CX,3
ENDIF
GET_2:	MOV	SI,DI			;  in a loop
	ADD	DI,2
	CALL	CHK_LINE
	LOOP	GET_2
	CMP	FLAG,0			;if non-zero, then we are doing the odd
	JZ	GET_3			;  scan lines at the end of the screen
	MOV	AL,FLAG 		;  to fill out the 8 bit words for the
	MOV	ZEROES,AL		;  printer.  Hence the zero fill
	MOV	FLAG,0			;reset flag for the main loop
GET_3:	
IF MPI
	MOV	CX,240
ELSE
	MOV	CX,320			;need to zero the screen bytes since
ENDIF
	MOV	DI,OFFSET SCRN		;  zero info from the screen is not
CLR_SCRN:				;  stored
	MOV	BYTE PTR [DI],0
	INC	DI
	LOOP	CLR_SCRN
	MOV	BL,LAST_COL		;bx = column pointer - work from right
	INC	BL
	XOR	BH,BH			;  to left as will skip trailing blanks
GET_3A: DEC	BX			;decrement first since last_col points
					;  to one below the window
IF MPI
	MOV	CX,3
ELSE
	MOV	CX,4			;process 4 scan lines except when have
ENDIF
	SUB	CL,ZEROES		;  extras at the bottom of the window
	MOV	BP,OFFSET ADDR		;bp points to scan line offsets
	MOV	SI,[BP] 		;  and si is the source index
	MOV	DI,OFFSET SCRN		;set up destination index
GET_4:	XOR	AL,AL			;clear AL
	TEST	BYTE PTR COLOR,4	;dump the green bank?
	JZ	GET_4A			;no, check the red bank
	OR	AL,ES:[SI][BX]		;yes, OR the byte into AL
GET_4A: TEST	BYTE PTR COLOR,2	;check the red bank
	JZ	GET_4B
	MOV	DX,0D000H
	MOV	ES,DX
	OR	AL,ES:[SI][BX]
GET_4B: TEST	BYTE PTR COLOR,1	;check the blue bank
	JZ	GET_4C
	MOV	DX,0C000H
	MOV	ES,DX
	OR	AL,ES:[SI][BX]
GET_4C: MOV	DX,0E000H		;restore ES to the green bank
	MOV	ES,DX
	CMP	LAST_BYTE,0		;have we found any non-zero data yet?
	JNZ	GET_4D			;yes, skip the check
	OR	AL,AL			;no, see if this is non-zero
	JZ	GET_4E			;if so, don't bother to store it
	MOV	LAST_BYTE,BL		;else set the last byte as this one,
	INC	LAST_BYTE		;  add one to it so gr_calc works ok,
GET_4D: MOV	[DI][BX],AL		;  and store all other screen bytes
GET_4E: ADD	BP,2			;bump scan line pointer
	MOV	SI,[BP] 		;  and get the next offset
	ADD	DI,80			;point to next row in the scrn array
	LOOP	GET_4			;  and do it again
	CMP	BL,FIRST_COL		;done with the whole line?
	JNZ	GET_3A			;no, do it again
	RET				;yes - return
GET_BYTES  ENDP
;
CHK_LINE  PROC	NEAR			;this routine checks to see if we need
	CMP	FLAG,0			; to add 128 or 1024 to the scan line
	JZ	CHK_0			; ptrs and checks to see if we have
	INC	FLAG			; completed all rows
	RET
CHK_0:	INC	S_LINE			;bump the scan line counter
	CMP	S_LINE,9		;end of a row?
	JNZ	CHK_2			;  yes->add 1024   no->add 128
	MOV	S_LINE,0		;yes also requires resetting s_line
	INC	ROW			;  and bumping the row counter
	MOV	AL,ROW
	DEC	AL
	CMP	AL,LAST_ROW		;all rows done?
	PUSHF
	INC	AL
	POPF
	JNZ	CHK_1			;no, continue
	INC	FLAG			;yes, incr flag and return
	RET
CHK_1:	MOV	AX,1024 		;new row, so add 1024
	JMP	CHK_3
CHK_2:	MOV	AX,128			;same row, so add 128
CHK_3:	ADD	AX,[SI] 		;increment the ptr
	MOV	[DI],AX 		;  and store it
	RET
CHK_LINE  ENDP
;
OUT_BYTES  PROC  NEAR			;routine to send 8 bytes to the prntr
					;a special routine is required since
					;  there is no special char to look
					;  for to stop
IF MX80 OR TI855 OR TI855WP OR IBM
	MOV	CX,4
ELSE
	MOV	CX,8			;set up counter
ENDIF
	MOV	BX,OFFSET PRNTR 	;base address
OUT_1:	MOV	AL,[BX] 		;the loop (same as PRN_OUT)
IF MPI
	SHR	AL,1
	SHR	AL,1
	OR	AL,40H
ENDIF
	CALL	FAR PTR BIOS_PRINT
	INC	BX
IF MX80 OR TI855 OR TI855WP OR IBM
	MOV	AL,[BX]
	CALL	FAR PTR BIOS_PRINT
	MOV	AL,[BX]
	CALL	FAR PTR BIOS_PRINT
	INC	BX
ENDIF
	LOOP	OUT_1
	RET
OUT_BYTES  ENDP
;
IF NOT MPI
GR_CALC PROC  NEAR
	XOR	AX,AX			;clear accumulator
	MOV	BL,FIRST_BYTE		;get the first col to dump and put
	MOV	AL,LAST_BYTE		;get the last col and
	SUB	AL,BL			;  find total number of cols +1
	SAL	AX,1			;now mult by 8 to get number of bytes
	SAL	AX,1
	MOV	DX,AX			;save *4 in dx
	SAL	AX,1
IF MX80 OR TI855 OR TI855WP OR IBM
	ADD	AX,DX
ENDIF
IF FX80 OR MX80 OR TI855 OR TI855WP OR IBM
	XOR	DX,DX			;clear dx for a 16 bit divide
	MOV	BX,256
	IDIV	BX			;find mod and remainder
	MOV	GR_STRG[GR_OFFSET],DL	;  and store in  GR_STRG
	MOV	GR_STRG[GR_OFFSET+1],AL
ENDIF
IF NEC
	MOV	SI,OFFSET DIVISORS	;point to first divisor
	MOV	CH,3			;set the count for # of divisors
	MOV	DI,OFFSET GR_STRG[3]	;and the dest ptr
	XOR	DH,DH
CALC_00:
	MOV	CL,-1			;init the count to -1
	MOV	DL,[SI] 		;get first divisor
CALC_01:
	MOV	BX,AX			;save ax in case we go negative
	SUB	AX,DX			;subtract a power of 10
	JC	CALC_02 		;if negative, then once too often
	INC	CL			;else inc count
	JMP	CALC_01 		;and do it again
CALC_02:
	MOV	AX,BX			;restore ax
	INC	CL			;inc count
	ADD	CL,30H			;add 30h to make it ascii
	MOV	[DI],CL 		;and store in the gr_strg
	INC	DI			;inc pointers
	INC	SI
	DEC	CH			;decrement divisor count
	JNZ	CALC_00 		;if not done, do it again
ENDIF
	RET
GR_CALC ENDP
ENDIF
;
INT_END LABEL	NEAR				;to ident end for the INT 27H
START:
	MOV	SP,OFFSET INT_STACK		;good place for it
	PUSH	DS
	XOR	AX,AX
	MOV	DS,AX				;clear DS
	MOV	SI,5*4				;SI points to interrupt
	CLI
	MOV	WORD PTR [SI],OFFSET INT_5
	MOV	WORD PTR [SI+2],CS		;set my interrupt vector
	STI
	POP	DS
	MOV	DX,OFFSET MSG
	MOV	AH,9
	INT	21H
	MOV	DX,OFFSET INT_END		;LWA of resident portion
	INT	27H				;terminate but stay resident
;
MSG	   DB	0DH,0AH,0AH,'Screen dump routine installed',0DH,0AH
IF FX80
	   DB	'Epson RX/FX, Star S-series'
ENDIF
IF MX80
	   DB	'Epson MX or Star'
ENDIF
IF IBM
	   DB	'IBM'
ENDIF
IF NEC
	   DB	'C. Itoh or NEC'
ENDIF
IF MPI
	   DB	'MPI'
ENDIF
IF TI855
	   DB	'TI-855 DP'
ENDIF
IF TI855WP
	   DB	'TI-855 WP'
ENDIF
	   DB	' Version 1.8, (c) 1984 Lindley Systems',0DH,0AH
	   DB	0AH,'$'
;
CODE	ENDS
	END	BEGIN
