	PAGE	,132
	TITLE	SCREENML - Screen Dump Utility	Version 1.7 (OKI 82/92 printer)
	.LFCOND
;
;	Kenneth A. Lively  26 March 1984
;	ML82 Update 22 June 84	RAL
;
;	SCREEN is a screen dump utility. It intercepts INT 5 from
;	the BIOS (generated when <shift f12> is struck), and dumps
;	the contents of the screen onto the PRN device.
;	Although any color combination can be dumped, depending on the
;	value of the variable COLOR, the program setscrn.com is respon-
;	sible for making sure that video ram is fully populated before
;	allowing a color other than green to be chosen.
;
;
FALSE	EQU	0
TRUE	EQU	NOT FALSE
ML82	EQU	TRUE		; For Microline 82/83
ML92	EQU	FALSE		; For Microline 92/93
BIOS_SEG SEGMENT AT 40H		; Segment where the BIOS is located
	ORG 4*3 		;   for ZDOS 1.1 and 2.0
BIOS_PRINT LABEL FAR	        ; Printer output
BIOS_SEG ENDS
;
CODE	SEGMENT	BYTE PUBLIC 'CODE'
	ASSUME	CS:CODE,DS:CODE,ES:CODE,SS:CODE
;
	ORG	100H
BEGIN:	JMP	START
;
	DW	VERSION
;
;	INT 5 handler
;

INT_5:	PUSH	DS			; Save his data segment
	PUSH	AX
	MOV	CS:INT_SP,SP
	MOV	CS:INT_SS,SS		; Save his stack
	MOV		AX,CS
	MOV	SS,AX			; Set local stack
	MOV	SP,OFFSET INT_STACK
	STI				; Now allow interrupts to come in
	PUSH	ES
	PUSH	BX
	PUSH	CX
	PUSH	DX
	PUSH	SI
	PUSH	DI
	MOV	DS,AX			; Set up my segments
	MOV	AX,0E000H
	MOV	ES,AX			; Note that ES points to green ram
	CALL	INT_INIT		; Initialize variables and printer
	CALL	INT_SCRN		; Process the screen
	POP	DI
	POP	SI
	POP	DX
	POP	CX
	POP	BX
	POP	ES
	CLI				; INTs off while messing around
	MOV	AX,CS:INT_SP
	MOV	SP,AX
	MOV	AX,CS:INT_SS		; Restore his stack
	MOV	SS,AX
	POP	AX
	POP	DS
	IRET				;back to BIOS
; data space
VERSION    DB	17			;current version 1/25/86
;The next 4 variables control the portion of the screen to be dumped and
;are not changed by the program.  This allows for a default screen dump
;of the entire screen.	Use SETSCRN to modify the memory resident code
;or change them under program control using the pointer at the INT 5 locn
;in absolute segment 0 to find their location.
FIRST_ROW  DB	0			;first row in the dump (starts at 0)
LAST_ROW   DB	24			;last row
FIRST_COL  DB	0			;first column in the dump (starts at 0)
LAST_COL   DB	79			;last column
COLOR	   DB	4			;default color is green bank
;
INT_SS	   DW	0			;saved stack values
INT_SP	   DW	0
ADDR	   DW	?
;
;The variable prntr contains the bytes from the screen in the high order
;bytes and the bytes to be sent to the printer in the low order bytes.
;
PRNTR	DW	9*25 dup (?)
;
;The following four variables are offsets from PRNTR to the indicated item
;
FIRST_BYTE DW	0			;first byte in any column
LAST_BYTE  DW	0			;last byte (+1) in any column
LAST	   DW	0			;first for the current column
FIRST	   DW	0			;last for the current column
COL	   DB	?			;current column
N_BYTES    DB	?			;this holds the no. of bytes in a col
N_ROWS	   DB	?			;no. of rows plus 1
B_SHFT	   DB	?			;the bit shift counter
MODE	   DB	?			;the CRT-C control byte storage
DONE	   DB	0			;flag to tell when done
CR_LF	   DB	03H,0EH,03H,02H 	;OKI graphics cr_lf and exit graphics
;
SET_25	   DB	1BH,'x1$'		;Clear 25th line
LD_STRG    DB	1CH			;set 12 cpi character width for 72 dpi
END_STRG   DB	1EH			;return to 10 cpi
MARGIN	   DB	1BH,'%B0000',03 	;set start print position (full column)
MARGIN1    DB	1BH,'%B0000',03 	;set start print position (partial col)
 IF ML82
MARGIN82   DB	'    ',3		;ML-82 MARGIN
 ENDIF
DIVISORS   DW	1000,100,10,1		;used for decimal division
;
	DW	128 DUP (?)
INT_STACK LABEL	NEAR			; Internal stack
;
INT_INIT  PROC	NEAR
	MOV	AH,9	; Clear 25th line if off, else does nothing
	MOV	DX,OFFSET SET_25
	INT	21H
	XOR	AX,AX			;set up scan line address to point to
	MOV	CL,FIRST_ROW		;  the first scan line of the first row
	OR	CL,CL			;zero means already on required row
	JZ	INIT_2
	XOR	CH,CH
INIT_1: ADD	AX,2048
	LOOP	INIT_1			;move to required start row
INIT_2: MOV	ADDR, AX		;save starting place for reuse
	MOV	AL,LAST_COL
	MOV	COL,AL			;set current column to last column
	MOV	AL,FIRST_ROW		;set first_byte and last_byte to the
	XOR	AH,AH			;  byte count from the top of the
	MOV	CL,18			;  screen to speed up things later.
	IMUL	CL
	MOV	FIRST_BYTE,AX
	MOV	DI,OFFSET MARGIN[3]	;set up the default start print
	CALL	CALC_START		;  string based on FIRST_ROW
	MOV	AL,LAST_ROW
	XOR	AH,AH
	INC	AL
	MOV	CL,18			;two dots per scan line
	IMUL	CL
	MOV	LAST_BYTE,AX
	SUB	AX,FIRST_BYTE		;now set n_bytes = to the no. of bytes
	SHR	AX,1			;divide by 2 to get actual count
	MOV	N_BYTES,AL
	MOV	BL,FIRST_ROW
	MOV	CL,LAST_ROW		;set NROWS = last_row - first_row + 1
	SUB	CL,BL
	INC	CL
	MOV	N_ROWS,CL
	MOV	B_SHFT,7		;init bit shift factor to 7
	MOV	DONE,0			;clear the done flag
 IFE ML82				;ML-82 only has 60 dots/inch
	MOV	BX,OFFSET LD_STRG	;set up 72 dots per inch
	MOV	CX,1
	CALL	PRN_OUT
 ENDIF
	IN	AL,0D8H 		;ensure that the cpu can access
	MOV	MODE,AL 		;  the video ram, but....
	AND	AL,7FH			;  keep a copy of the original
	OUT	0D8H,AL 		;  so we can put it back
	RET
INT_INIT  ENDP
;
;This is the main routine for processing the screen dump
;
INT_SCRN  PROC	NEAR
SCRN_0: MOV	AL,COL			;get current column
	CMP	AL,FIRST_COL		;are we done?
	JNL	SCRN_1			;no, continue
	MOV	DONE,1			;signal that this is the last
SCRN_1: XOR	AH,AH
	MOV	SI,ADDR 		;get offset to first scan line
	ADD	SI,AX			;  and add in current column
	MOV	DI,OFFSET PRNTR[1]	;point to first position of PRNTR
	MOV	AX,FIRST_BYTE		;  words where we'll put data
	ADD	DI,AX
	MOV	CL,N_ROWS		;get the no. of rows to process
	XOR	CH,CH
	CALL	GET_BYTES		;get a column's worth of bytes
	DEC	COL			;decrement the column index
	MOV	BL,B_SHFT		;if bshft = 0, then already have 7
	OR	BL,BL			;  bits in the low order byte
	JZ	SCRN_2A
;now shift the bytes from the screen right the appropriate number of bits
;  to fill in the bytes to be sent to the printer (need 7 bits)
SCRN_1A:
	MOV	CL,N_BYTES		;the no. of bytes to process
	XOR	CH,CH
	MOV	DI,OFFSET PRNTR 	;point to the first screen byte
	MOV	AX,FIRST_BYTE
	ADD	DI,AX
SCRN_2: PUSH	CX			;save loop counter
	MOV	CL,BL			;get the shift factor to CL
	SHR	WORD PTR [DI],CL	;  and do it
	ADD	DI,2			;bump pointer
	POP	CX			;recover count
	LOOP	SCRN_2			;continue for n_bytes times
;It turns out that the new value for b_shft is always (8 - b_shft) at this
;point and (7 - b_shft) after the remaining bits have been shifted in,
;except that every seven times, we will already have the 7 bits we need and
;this part is skipped.
SCRN_2A:
	MOV	BH,8			;set up shift factor to shift remaining
	SUB	BH,BL			;  bits to the low order byte
	MOV	B_SHFT,BH		;  and store
	CALL	FIND			;find first and last non-zero info
	JNC	SCRN_3			;if line is blank, do a cr_lf,
	MOV	AL,03H			; but have to be in graphics mode first
	CALL	FAR PTR BIOS_PRINT
	JMP	SHORT SCRN_5
SCRN_3: MOV	DX,LAST 		;calculate the no. of bytes to send
	MOV	AX,FIRST		; = (last - first)/2 + 1
	SUB	DX,AX
	SHR	DX,1
	INC	DX
 IF ML82
	MOV	CX,5
	MOV	BX,OFFSET MARGIN82
 ENDIF
 IF ML92
        MOV	BX,OFFSET MARGIN
	CMP	AX,FIRST_BYTE		;if FIRST = FIRST_BYTE, use the
	JZ	SCRN_4			;  precalculated start position
	PUSH	DX
	MOV	DI,OFFSET MARGIN1[3]
	CALL	CALC_START		;else calculate the new one
	POP	DX
	MOV	BX,OFFSET MARGIN1	;point to string modified by calc_start
	JMP	SHORT SCRN_4A
SCRN_4: MOV	CX,8
 ENDIF
	CALL	PRN_OUT 		;set start print position, enter graphic
	CALL	OUT_BYTES		;mode, and send out the screen bytes
SCRN_5:
	MOV	CX,4			;send the cr_lf and exit graphics mode
	MOV	BX,OFFSET CR_LF
	CALL	PRN_OUT
	CMP	DONE,1
	JZ	SCRN_DONE
	MOV	BL,B_SHFT		;check to see if bshft needs to be reset
	CMP	BL,8
	JNZ	SCRN_5A
	MOV	B_SHFT,7		;yes, reset it to 7, skip this shift,
	MOV	BL,B_SHFT		;  and don't get screen bytes this time
	JMP	SCRN_1A
;now shift the remaining bits from the high order byte to the low order byte
;to make room for the next column of screen bytes
SCRN_5A:
	MOV	CL,N_BYTES		;the no. of bytes to work with
	XOR	CH,CH
	MOV	DI,OFFSET PRNTR 	;point to first word in screen dump
	MOV	AX,FIRST_BYTE
	ADD	DI,AX
SCRN_6: PUSH	CX			;save loop counter
	MOV	CL,BL			;get the shift factor to CL
	SHR	WORD PTR [DI],CL	;  and do it
	ADD	DI,2			;bump pointer
	POP	CX			;recover count
	LOOP	SCRN_6			;continue for n_bytes times
	MOV	BH,7			;set up shift factor to shift in 7
	SUB	BH,BL			;  bits the next time
	MOV	B_SHFT,BH		;  and store it
	JMP	SCRN_0			;start the next line
SCRN_DONE:				;arrive here when done with the screen
 IFE ML82
	MOV	BX,OFFSET END_STRG	;point to string to reset printer
	MOV	CX,1
	CALL	PRN_OUT 		;  and send it out
 ENDIF
	MOV	AL,MODE 		;all done, so put back the control
	OUT	0D8H,AL 		;  word at port D8
	RET				;  and return
INT_SCRN  ENDP
;
;Find the first and last non-zero bytes in the current colummn of data to avoid
;sending a whole line when only part of it contains graphics data.
;
FIND	PROC NEAR
	MOV	BX,FIRST_BYTE		;get offset to first_byte
	MOV	SI,OFFSET PRNTR 	;only check the low order bytes
FIND_1: CMP	BYTE PTR [SI][BX],0	;zero data?
	JZ	FIND_2			;yes, check next scan line of column
	MOV	FIRST,BX		;no, set first to this one, and check
	JMP	SHORT FIND_3		;  the end of the column
FIND_2: ADD	BX,2			;point to next screen byte
	CMP	BX,LAST_BYTE		;have we reached the last byte?
	JNZ	FIND_1			;no, continue
	STC				;yes, set carry flag to show blank line
	RET
FIND_3: 
 IF ML82
	MOV	BX,FIRST_BYTE		;go back to beginning byte
	MOV	FIRST,BX
 ENDIF
	MOV	BX,LAST_BYTE		;now start at the end and work backwards
FIND_4: SUB	BX,2			;subtract first
	CMP	BYTE PTR [SI][BX],0
	JZ	FIND_4			;if zero, decrement and check again
	MOV	LAST,BX 		;else set last as this one,
	CLC				;  clear carry flag, and return
	RET
FIND	ENDP
;
;A general purpose routine to send cx bytes to the printer. BX contains offset
;
PRN_OUT  PROC	NEAR
P_LOOP: MOV	AL,[BX]
	CALL	FAR PTR BIOS_PRINT
	INC	BX
	LOOP	P_LOOP
	RET
PRN_OUT  ENDP
;
;This subroutine fetches the bytes from video ram for the current column.
;The appropriate banks are or'd together based on the value of COLOR.
;Registers on entry are:
;    SI points to the 1st scan line of the 1st row of the current column
;    DI points to the first storage position for the column image
;    CX contains the number of rows to process
;
;When done = 1, this routine performs a zero fill for the first column
;
GET_BYTES  PROC  NEAR
GET_1:	PUSH	CX			;save the row counter
	MOV	CX,9			;  and set the scan line counter
GET_2:	XOR	AX,AX			;clear AX so we can OR all data into it
	CMP	DONE,1			;just do a zero fill if done flag is set
	JZ	GET_2D
	TEST	BYTE PTR COLOR,4	;dump the green bank?
	JZ	GET_2A			;no, check the red bank
	OR	AL,ES:[SI]		;yes, OR the byte into AL
GET_2A: TEST	BYTE PTR COLOR,2	;check the red bank
	JZ	GET_2B
	MOV	DX,0D000H
	MOV	ES,DX
	OR	AL,ES:[SI]
GET_2B: TEST	BYTE PTR COLOR,1	;check the blue bank
	JZ	GET_2C
	MOV	DX,0C000H
	MOV	ES,DX
	OR	AL,ES:[SI]
GET_2C: MOV	DX,0E000H		;restore ES to the green bank
	MOV	ES,DX
GET_2D: MOV	[DI],AL 		;store the screen byte
;now update all the pointers and do the next scan line
	ADD	DI,2			;bump PRNTR byte pointer
	ADD	SI,128			;point to next scan line
	LOOP	GET_2			;  and do it again
;increment scan line pointer to the next row and decrement row counter
	POP	CX
	ADD	SI,896			;increment to pt to next row
	LOOP	GET_1
	RET
GET_BYTES  ENDP
;
;This subroutine sends the bytes to the printer.  Each byte is sent twice
;in order to give the proper aspect ratio.  Any 03H bytes also need to be
;sent twice so the OKI doesn't ignore them. DX contains count
;
OUT_BYTES  PROC  NEAR
	MOV	CX,DX			;set up counter
	MOV	BX,OFFSET PRNTR 	;base address
	ADD	BX,FIRST		;  plus staring byte
OUT_1:	MOV	AL,[BX] 		;the loop (same as PRN_OUT)
	SHR	AL,1			;shift to lower 7 bits
	CALL	FAR PTR BIOS_PRINT	;send this byte twice to the printer
	CALL	FAR PTR BIOS_PRINT
	CMP	AL,03H			;if a 03H, send it two more times so
	JNZ	OUT_2			;  that 2 + 2 = 2 (!)
	CALL	FAR PTR BIOS_PRINT
	CALL	FAR PTR BIOS_PRINT
OUT_2:	ADD	BX,2			;increment pointer
	LOOP	OUT_1
	RET
OUT_BYTES  ENDP
;		
;This subroutine calculates the start print position for the current column
;of graphics data.
;
CALC_START PROC  NEAR
	ADD	AX,63			;add in left margin
	SHL	AX,1			;times 2 to get correct start position
	MOV	SI,OFFSET DIVISORS	;point to first divisor
	MOV	CH,4			;set the count for # of divisors
CALC_00:
	MOV	CL,-1			;init the count to -1
	MOV	DX,[SI] 		;get first divisor
CALC_01:
	MOV	BX,AX			;save ax in case we go negative
	SUB	AX,DX			;subtract a power of 10
	JC	CALC_02 		;if negative, then once too often
	INC	CL			;else inc count
	JMP	CALC_01 		;and do it again
CALC_02:
	MOV	AX,BX			;restore ax
	INC	CL			;inc count
	ADD	CL,30H			;add 30h to make it ascii
	MOV	[DI],CL 		;  and store in the margin string
	INC	DI			;inc pointers
	ADD	SI,2
	DEC	CH			;decrement divisor count
	JNZ	CALC_00 		;if not done, do it again
	RET
CALC_START ENDP
;
INT_END LABEL	NEAR				;to ident end for the INT 27H
START:
	MOV	SP,OFFSET INT_STACK		;good place for it
	PUSH	DS
	XOR	AX,AX
	MOV	DS,AX				;clear DS
	MOV	SI,5*4				;SI points to interrupt
	CLI
	MOV	WORD PTR [SI],OFFSET INT_5
	MOV	WORD PTR [SI+2],CS		;set my interrupt vector
	STI
	POP	DS
	MOV	DX,OFFSET MSG
	MOV	AH,9
	INT	21H
	MOV	DX,OFFSET INT_END		;LWA of resident portion
	INT	27H				;terminate but stay resident
;
MSG	   DB	0DH,0AH,0AH,'Graphics Screen copy for Microline '
 IF ML82
	   DB	'82/83'
 ENDIF
 IF ML92
	   DB	'92/93'
 ENDIF
	   DB	' installed',0DH,0AH
	   DB	'Lindley Systems Version 1.7	 '
	   DB	'(C)1984 Kenneth A. Lively',0DH,0AH,'$'
CODE	ENDS
	END	BEGIN
